@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -337,7 +337,7 @@ bool Basic_buffer::piseow( const Point & p ) const
 
 Point Basic_buffer::bol( const Point & p ) const
   {
-  if( p.line >= lines() ) return eof();
+  if( p.line >= lines() ) return Point( last_line(), 0 );
   if( p.line < 0 ) return bof();
   return Point( p.line, 0 );
   }
@@ -374,18 +374,19 @@ Point Basic_buffer::eot( const Point & p ) const
 // begin of paragraph (bol of first non-blank line in paragraph)
 Point Basic_buffer::bop( const Point & p ) const
   {
-  Point p1 = p;
+  Point p1 = bol( p );
   if( !blank( p1.line ) ) while( !blank( p1.line - 1 ) ) --p1.line;
-  return bol( p1 );
+  return p1;
   }
 
 
-// end of paragraph (bol of blank line after paragraph)
+// end of paragraph (bol of blank line after paragraph, or eof)
 Point Basic_buffer::eop( const Point & p ) const
   {
-  Point p1 = p;
+  Point p1 = bol( p );
   while( !blank( p1.line ) ) ++p1.line;
-  return bol( p1 );
+  if( p1.line >= lines() ) return eof();
+  return p1;
   }
 
 
@@ -609,22 +610,19 @@ Point Basic_buffer::to_cursor( const Point & pointer ) const
 
 Point Basic_buffer::to_pointer( const Point & cursor, Point & pcursor ) const
   {
-  pcursor = bol( cursor );
-  Point pointer = pcursor;
-  if( pointer.line >= 0 && pointer.line < lines() )
+  if( cursor.line >= lines() ) { pcursor = eof(); return pcursor; }
+  Point pointer = pcursor = bol( cursor );
+  const std::string & s = data[pointer.line];
+  const long peolcol = eol( pointer ).col;
+  while( pcursor.col < cursor.col && pointer.col < peolcol )
     {
-    const std::string & s = data[pointer.line];
-    const long peolcol = eol( pointer ).col;
-    while( pcursor.col < cursor.col && pointer.col < peolcol )
+    if( s[pointer.col] == '\t' )
       {
-      if( s[pointer.col] == '\t' )
-        {
-        long col = pcursor.col + 8 - ( pcursor.col % 8 );
-        if( col <= cursor.col ) pcursor.col = col; else break;
-        }
-      else ++pcursor.col;
-      ++pointer.col;
+      const long col = pcursor.col + 8 - ( pcursor.col % 8 );
+      if( col <= cursor.col ) pcursor.col = col; else break;
       }
+    else ++pcursor.col;
+    ++pointer.col;
     }
   return pointer;
   }
