@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -53,6 +53,8 @@ Point & Block::begin() { return begin_; }
 
 Point & Block::end() { return end_; }
 
+long Block::last_mline() { return end_.line - ( end_.col <= 0 ); }
+
 
 bool Block::in_block( const Buffer & buffer, const Point & p )
   {
@@ -165,7 +167,7 @@ bool Block::copy_block( Buffer & buffer, const Point & p, const bool move )
       !bufferp_->pisvalid( begin_ ) || !bufferp_->pisvalid( end_ ) )
     return done;
   Point p_end;
-  const bool same = ( &buffer == bufferp_ );
+  const bool same = &buffer == bufferp_;
   buffer.reset_appendable();
   if( !RC::editor_options().rectangle_mode )
     {
