@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,8 +21,9 @@ namespace Block {
 
 Buffer * bufferp();
 Point & anchor();		// anchor for marking
-Point & begin();
+Point & begin();		// pointers into *bufferp
 Point & end();
+long last_mline();		// last line visibly marked (col > 0)
 
 bool in_block( const Buffer & buffer, const Point & p );
 bool in_block_or_end( const Buffer & buffer, const Point & p );
