@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <string>
 #include <vector>
 #include <unistd.h>
+#include <utime.h>
 #include <sys/stat.h>
 
 #include "buffer.h"
@@ -60,9 +61,9 @@ void adjust_point( const Buffer & b, Point & p, const Point & p1,
                    const Point & p2_old, const Point & p2_new )
   {
   if( p <= p1 ) return;			// was before the block, ignore it
-  if( p < p2_old )			// was in the block, keep in
+  if( p < p2_old )			// was in the block, keep in or at end
     {
-    if( p >= p2_new ) { p = p2_new; if( p > p1 ) b.pprev( p ); }
+    if( p >= p2_new ) p = p2_new;
     else if( !b.pisvalid( p ) ) b.pvalid( p );
     }
   else					// was beyond the block
@@ -85,6 +86,24 @@ void adjust_marks_and_pointers( Buffer & b, const Point & p1,
   }
 
 
+/* Return the number of bytes really read.
+   If (value returned < size) and (errno == 0), means EOF was reached.
+*/
+long freadblock( char * const buf, const long size, FILE * const f )
+  {
+  long sz = 0;
+  errno = 0;
+  while( sz < size )
+    {
+    const long n = std::fread( buf + sz, 1, size - sz, f );
+    if( n > 0 ) sz += n;
+    else if( errno != EINTR ) break;
+    errno = 0;
+    }
+  return sz;
+  }
+
+
 /* Return the number of bytes really written.
    If (value returned < size), it is always an error.
 */
@@ -123,6 +142,31 @@ bool write_to_file( const std::vector< std::string > & data,
   return true;
   }
 
+
+bool copy_file( const char * const name, const char * const backname,
+                struct stat * const stp )
+  {
+  FILE * f = std::fopen( name, "r" ); if( !f ) return false;
+  FILE * fb = std::fopen( backname, "w" );
+  if( !fb ) { std::fclose( f ); return false; }
+  enum { bufsize = 65536 };
+  char buf[bufsize];
+  bool error = false;
+
+  while( !error )
+    {
+    const int rd = freadblock( buf, bufsize, f );
+    if( ( rd != bufsize && errno ) ||
+        ( rd > 0 && fwriteblock( buf, rd, fb ) != rd ) ) error = true;
+    if( rd < bufsize ) break;				// EOF
+    }
+  if( ( std::fclose( fb ) != 0 ) | ( std::fclose( f ) != 0 ) ) error = true;
+  if( !error && ( !S_ISLNK( stp->st_mode ) || stat( name, stp ) == 0 ) )
+    { struct utimbuf t; t.actime = stp->st_atime; t.modtime = stp->st_mtime;
+      utime( backname, &t ); }
+  return !error;
+  }
+
 } // end namespace
 
 
@@ -216,8 +260,8 @@ bool Buffer::Options::set_lmargin( const int l )
 
 bool Buffer::Options::set_rmargin( const int r )
   {
-  if( r > lmargin_ && r < INT_MAX / 2 ) { rmargin_ = r; return true; }
-  if( r == -1 ) { rmargin_ = INT_MAX / 2 - 1; return true; }
+  if( r > lmargin_ && r < 99999999 ) { rmargin_ = r; return true; }
+  if( r == -1 ) { rmargin_ = 99999999 - 1; return true; }
   return false;
   }
 
@@ -230,7 +274,7 @@ Buffer::Buffer( const char * const namep, const bool read_only )
     existed( false )
   {
   FILE * f = 0;
-  const bool valid_namep = ( namep && namep[0] );
+  const bool valid_namep = namep && namep[0];
   const bool is_stdin = !namep || ( namep[0] == '-' && namep[1] == 0 );
   errno = 0;
   if( is_stdin && !isatty( fileno( stdin ) ) ) f = stdin;
@@ -243,8 +287,8 @@ Buffer::Buffer( const char * const namep, const bool read_only )
     }
   else return;				// empty unnamed buffer
 
-  if( !f )
-    { if( errno == ENOENT ) return; else throw Error( "Error opening file" ); }
+  if( !f ) { if( errno == ENOENT ) return;
+             throw Error( "Error opening file for reading" ); }
   if( isatty( fileno( f ) ) )
     { std::fclose( f ); throw Error( "Is a terminal" ); }
 
@@ -324,8 +368,8 @@ int Buffer::set_marks()		// return the number of new marks set
 bool Buffer::save( const std::string * namep, bool save_to,
                    const bool append, Point p1, Point p2 ) const
   {
-  const bool valid_namep = ( namep && namep->size() );
-  const bool had_name = ( name_.size() > 0 );
+  const bool valid_namep = namep && namep->size();
+  const bool had_name = name_.size();
   if( !valid_namep && !had_name ) return false;		// no names
   if( !valid_namep && save_to ) return false;		// no name to save to
 
@@ -365,19 +409,27 @@ bool Buffer::save( const std::string * namep, bool save_to,
       throw Error( "Error creating intermediate directory" ); }
   if( !save_to && !saved && !append )
     {
+    const bool preserve_links = RC::editor_options().preserve_links &&
+                                lstat( namep->c_str(), &st ) == 0 &&
+                                ( S_ISLNK( st.st_mode ) || st.st_nlink > 1 );
     if( RC::editor_options().backup && !backuped )
       {
-      std::string backname( *namep ); backname += '~';
-      std::remove( backname.c_str() );
-      std::rename( namep->c_str(), backname.c_str() );
+      std::string s( *namep ); s += '~';
+      const char * const backname = s.c_str();
+      std::remove( backname );
+      if( ( ( !preserve_links && std::rename( namep->c_str(), backname ) ) ||
+            ( preserve_links && !copy_file( namep->c_str(), backname, &st ) ) ) &&
+          errno != ENOENT )
+        { if( !had_name ) name_.clear();
+          throw Error( "Error making backup copy" ); }
       backuped = true;
       }
-    else std::remove( namep->c_str() );		// avoid following links
+    else if( !preserve_links ) std::remove( namep->c_str() );	// break link
     }
 
   FILE * f = std::fopen( namep->c_str(), append ? "a" : "w" );
-  if( !f )
-    { if( !had_name ) { name_.clear(); } throw Error( "Error opening file" ); }
+  if( !f ) { if( !had_name ) { name_.clear(); }
+             throw Error( "Error opening file for writing" ); }
   if( mode ) fchmod( fileno( f ), mode );
 
   if( !this->pisvalid( p1 ) ) p1 = this->bof();
@@ -451,7 +503,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
     if( options.auto_indent )
       {
       const Point pt = bot( p );
-      if( pt.col > 0 && p.col >= pt.col )
+      if( pt.col > 0 && pt.col <= p.col )
         tmp.pputb( dummy, *this, bol( pt ), pt );
       }
     const bool result = pputb( p, tmp, tmp.bof(), tmp.eof(), true );
@@ -497,6 +549,7 @@ bool Buffer::pputc( Point & p, const unsigned char ch )
         while( p1 > pt && pprev( p1 ) && ISO_8859::isblank( (*this)[p1] ) );
         if( p1 >= pt && p1 < p2 && pnext( p1 ) && pnext( p2 ) )
           {
+          modified_ = true;
           Point p3 = p1;
           reset_appendable(); pdelb( p1, p2 );
           force_append(); pputc( p3, '\n' );
@@ -678,14 +731,14 @@ bool Buffer::reformat_paragraph( Point & cursor, const bool append_change )
           if( period2 ) tmp.pputc( dummy, ' ' ); }
       else if( pending_nbsp )
         { pending_nbsp = false; tmp.pputc( dummy, 0xA0 ); }
-      tmp.pputc( dummy, ch ); // if( dummy != tmp.eof() ) std::exit( 3 );
-      period2 = ( ch == '.' && options.two_spaces );
+      tmp.pputc( dummy, ch );
+      period2 = ch == '.' && options.two_spaces;
       }
     if( Screen::kbhit() >= 0 && Screen::wait_kbhit() == 3 )
       throw Error( "Aborted" );
     }
   if( dummy != tmp.bol( dummy ) ) tmp.pputc( dummy, '\n' );
-  if( !change ) change = ( tmp.last_line() != ( eop.line - bop.line ) );
+  if( !change ) change = tmp.last_line() != eop.line - bop.line;
   if( !change )
     for( long line = 0; line < tmp.lines(); ++line )
       if( tmp.characters( line ) != characters( bop.line + line ) &&
