@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -106,22 +106,22 @@ void Bufhandle_vector::init()
 const char * Bufhandle_vector::save_all_named()
   {
   int count = 0;
-  bool done = true;
+  bool all = true;
 
   for( int i = 0; i < handles(); ++i )
     {
     const Buffer & buffer = data[i].buffer();
     if( buffer.modified() && buffer.name().size() )
       {
-      try { if( buffer.save() ) ++count; else done = false; }
-      catch( Buffer::Error & ) { done = false; }
+      try { if( buffer.save() ) ++count; else all = false; }
+      catch( Buffer::Error & ) { all = false; }
       }
     }
   Screen::show_status_lines();
   int bufsize = 0;
   char * const buf = Screen::msg_line_buf( &bufsize );
-  snprintf( buf, bufsize, "%s%d named buffer%s saved",
-            done ? "" : "Only ", count, ( count == 1 ) ? "" : "s" );
+  snprintf( buf, bufsize, "%s%s named buffer%s saved", all ? "" : "Only ",
+            format_num3( count ), ( count == 1 ) ? "" : "s" );
   return buf;
   }
 
@@ -144,8 +144,8 @@ const char * Bufhandle_vector::show_status()
   int bufsize = 0;
   char * const buf = Screen::msg_line_buf( &bufsize );
   snprintf( buf, bufsize,
-            "**  %d modified and  %d unnamed out of  %d total buffers **",
-            mcount, ucount, bcount );
+            "**  %s modified and  %s unnamed out of  %s total buffers **",
+            format_num3( mcount ), format_num3( ucount ), format_num3( bcount ) );
   return buf;
   }
 
