@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -539,6 +539,10 @@ int map_to_byte( const int code )
     case 0x0585: return 'o';	// armenian small letter oh
     case 0x0586: return 'f';	// armenian small letter feh
     case 0x060C: return ',';	// arabic comma
+    case 0x066A: return '%';	// arabic percent sign
+    case 0x066B: return '.';	// arabic decimal separator
+    case 0x066C: return '\'';	// arabic thousands separator
+    case 0x066D: return '*';	// arabic five pointed star
     case 0x10D0: return 'a';	// georgian letter an
     case 0x10D1: return 'b';	// georgian letter ban
     case 0x10D2: return 'g';	// georgian letter gan
@@ -774,6 +778,7 @@ int map_to_byte( const int code )
     case 0x2501: return '-';	// box drawings heavy horizontal
     case 0x2502:		// box drawings light vertical
     case 0x2503: return '|';	// box drawings heavy vertical
+    case 0x2514: return '`';	// box drawings light up and right
     case 0x2571: return '/';	// box drawings light diagonal upper right
     case 0x2572: return '\\';	// box drawings light diagonal upper left
     case 0x27E8: return '<';	// mathematical left angle bracket
@@ -785,6 +790,7 @@ int map_to_byte( const int code )
   if( code < 256 ) return code;
 //  if( code >= 0x02B0 && code <= 0x02FF ) return 256;	// spacing modifier letters
 //  if( code >= 0x0300 && code <= 0x036F ) return 256;	// combining diacritical marks
+  if( code >= 0x0660 && code <= 0x0669 ) return code - 0x0660 + '0';	// arabic-indic digits
   if( ( code >= 0x2000 && code <= 0x200B ) ||
       code == 0x202F || code == 0x205F || code == 0x3000 ) return ' ';
   if( code == 0x200C || code == 0x200D ) return 256;	// zero width (non-)joiner
@@ -936,8 +942,8 @@ void Encoding::base64_encode( const std::string & in, std::string & out )
   out.clear();
   for( unsigned long i = 0; i < in.size(); i += 3 )
     {
-    const bool s1 = ( i + 1 < in.size() );
-    const bool s2 = ( i + 2 < in.size() );
+    const bool s1 = i + 1 < in.size();
+    const bool s2 = i + 2 < in.size();
     const unsigned char c0 = in[i];
     const unsigned char c1 = s1 ? in[i+1] : 0;
     const unsigned char c2 = s2 ? in[i+2] : 0;
@@ -950,7 +956,7 @@ void Encoding::base64_encode( const std::string & in, std::string & out )
 
 
 /* Decode base64 encoded input string 'in' to output string 'out'.
-   Return true if decoding was successful, i.e. if the input was valid
+   Return true if decoding was successful, i.e., if the input was valid
    base64 data. Note that as soon as any invalid character is found,
    decoding is stopped, the index of the invalid 4-byte group is stored in
    *idxp, and false is returned. This means that any line terminators must
@@ -991,7 +997,7 @@ bool Encoding::base64_decode( const std::string & in, std::string & out,
 
 
 /* Decode quoted-printable encoded input string 'in' to output string 'out'.
-   Return 0 if decoding was successful, i.e. if the input was valid
+   Return 0 if decoding was successful, i.e., if the input was valid
    quoted-printable data. Note that as soon as any invalid character is
    found, decoding is stopped, the index of the invalid character is stored
    in *idxp, and false is returned.
@@ -1097,7 +1103,7 @@ unsigned char Encoding::rot47( const unsigned char ch )
 
 
 /* Encode ISO-8859-15 encoded input string 'in' to output string 'out'.
-   Return false if no encoding is needed, i.e. if the input is already
+   Return false if no encoding is needed, i.e., if the input is already
    valid ASCII data. Any ASCII text in 'in' is copied unmodified.
 */
 bool Encoding::ascii_encode( const std::string & in, std::string & out )
@@ -1138,7 +1144,7 @@ void Encoding::ucs_to_utf8( const int code, std::string & out )
 
 
 /* Encode ISO-8859-15 encoded input string 'in' to output string 'out'.
-   Return false if no encoding is needed, i.e. if the input is already
+   Return false if no encoding is needed, i.e., if the input is already
    valid UTF-8 data.
    If force is true and any character is >= 128, encoding is performed
    unconditionally.
@@ -1197,7 +1203,7 @@ int Encoding::utf8_to_ucs( const std::string & seq, const unsigned long i,
 
 
 /* Decode UTF-8 encoded input string 'in' to output string 'out'.
-   Return 0 if decoding was successful, i.e. if the input was valid UTF-8
+   Return 0 if decoding was successful, i.e., if the input was valid UTF-8
    data in the ISO-8859-[1|15] range. Note that as soon as any invalid
    character is found, decoding is stopped, the index of the invalid
    character is stored in *idxp, and:
@@ -1222,6 +1228,7 @@ int Encoding::utf8_decode( const std::string & in, std::string & out,
     if( str && code >= 0xBC && code <= 0xBE && !out.empty() &&
         std::isdigit( (unsigned char)out.end()[-1] ) ) out += ' ';
     if( str ) out += str;
+    else if( code == 0xFEFF ) {}	// remove UTF-8 Byte Order Mark (BOM)
     else if( force ) out.append( in, i, len );
     else { if( idxp ) { *idxp = i; } return code; }
     i += len;
@@ -1247,7 +1254,7 @@ int Encoding::remove_utf8_out_of_range( const std::string & in, std::string & ou
     int len;
     const int code = utf8_to_ucs( in, i, &len );
     if( code < 0 ) { if( idxp ) { *idxp = i; } return -1; }
-    if( map_to_byte( code ) >= 0 || map_to_string( code ) )
+    if( map_to_byte( code ) >= 0 || map_to_string( code ) || code == 0xFEFF )
       out.append( in, i, len );
     else modified = true;
     i += len;
