@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -146,6 +146,27 @@ unsigned char xdigit( const unsigned value )	// hex digit for 'value'
 } // end namespace ISO_8859
 
 
+// separate hex numbers of 5 or more digits in groups of 4 digits using '_'
+const char * format_hex4( unsigned long long num )
+  {
+  enum { buffers = 8, bufsize = 3 * sizeof num };
+  static char buffer[buffers][bufsize];	// circle of buffers for printf
+  static int current = 0;
+
+  char * const buf = buffer[current++]; current %= buffers;
+  char * p = buf + bufsize - 1;		// fill the buffer backwards
+  *p = 0;				// terminator
+
+  for( int i = 0; ; )
+    {
+    *(--p) = ISO_8859::xdigit( num & 0x0F ); num >>= 4;
+    if( num == 0 ) break;
+    if( ++i >= 4 ) { i = 0; *(--p) = '_'; }
+    }
+  return p;
+  }
+
+
 /* 'seq[i]' begins a escape sequence (the characters following a '\').
    Return the corresponding code and, in *lenp, the number of bytes read.
    Return -1 if unknown escape sequence, -2 if syntax error.
@@ -568,30 +589,8 @@ void ISO_8859::map_to_ascii( const unsigned char ch, std::string & out )
 
 int ISO_8859::xvalue( const unsigned char ch )
   {
-  switch( ch )
-    {
-    case '0': return 0;
-    case '1': return 1;
-    case '2': return 2;
-    case '3': return 3;
-    case '4': return 4;
-    case '5': return 5;
-    case '6': return 6;
-    case '7': return 7;
-    case '8': return 8;
-    case '9': return 9;
-    case 'a':
-    case 'A': return 0x0A;
-    case 'b':
-    case 'B': return 0x0B;
-    case 'c':
-    case 'C': return 0x0C;
-    case 'd':
-    case 'D': return 0x0D;
-    case 'e':
-    case 'E': return 0x0E;
-    case 'f':
-    case 'F': return 0x0F;
-    default: return -1;
-    }
+  if( ch >= '0' && ch <= '9' ) return ch - '0';
+  if( ch >= 'A' && ch <= 'F' ) return ch - 'A' + 10;
+  if( ch >= 'a' && ch <= 'f' ) return ch - 'a' + 10;
+  return -1;
   }
