@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,8 +41,11 @@ inline bool isblank( const unsigned char ch )
 inline bool isspace( const unsigned char ch )	// is 'ch' whitespace?
   { return std::isspace( ch ) || ch == 0xA0; }
 
+/* Don't consider characters in control block C1 (0x80-0x9F) as control chars
+   so that UTF-8 file names can be copied from the buffer to the file prompt
+   and typed in the buffer and in the file prompt. */
 inline bool iscntrl( const unsigned char ch )
-  { return ch < 32 || ( ch >= 0x7F && ch < 0xA0 ); }
+  { return ch < 32 || ch == 0x7F; }		// 0x80-0x9F appear in UTF-8
 inline bool isgraph( const unsigned char ch )
   { return !iscntrl( ch ) && ch != ' ' && ch != 0xA0; }
 inline bool isprint( const unsigned char ch )
@@ -73,3 +76,5 @@ inline bool equivalent( const unsigned char ch1, const unsigned char ch2,
   }
 
 } // end namespace ISO_8859
+
+const char * format_hex4( unsigned long long num );
