@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,65 +43,68 @@ namespace {
 
 const char * const     program_name = "moe";
 const char * const config_file_name = "moe.conf";
-const char * const     program_year = "2025";
+const char * const     program_year = "2026";
 const char * invocation_name = program_name;		// default value
 
 
 void show_help()
   {
-  std::printf( "GNU moe is a console text editor for ISO-8859 and ASCII character encodings.\n"
-               "It has a modeless, user-friendly interface, online help, multiple windows,\n"
-               "global search/replace (on all buffers at once), block operations, automatic\n"
-               "indentation, word wrapping, file name completion, directory browser,\n"
-               "duplicate removal from prompt histories, delimiter matching, text conversion\n"
-               "from/to UTF-8, romanization, etc. The file size, line length, number of\n"
-               "buffers, and undo/redo capability are only limited by the amount of memory\n"
-               "available and the size of the address space of your machine.\n"
-               "\nMoe respects your work. By default it won't automatically add, change, or\n"
-               "remove a single byte in your files. Moe is a WYTIWYG (what you type is what\n"
-               "you get) editor.\n"
-               "\nMoe can easily edit thousands of files at the same time.\n"
-               "\nUsage: %s [global_options] [[+[line[,col]]] file_name [file_options]]...\n",
-               invocation_name );
-  std::printf( "\n'global_options' are a mix of editor options and default 'file_options'.\n"
-               "Most long option names have a negative form: '--backup' '--no-backup'.\n"
-               "\nEach file name may be preceded by '+[line[,column]]' to start the cursor\n"
-               "at the line, and optionally column, specified. 'line' and 'column' are\n"
-               "applied to subsequent files until new values are specified for them.\n"
-               "A '+' alone puts the cursor at end-of-buffer.\n"
-               "\nA hyphen '-' used as a file name argument means standard input.\n"
-               "Directories are recursively descended into.\n"
-               "\nEditor options:\n"
-               "  -h, --help               display this help and exit\n"
-               "  -V, --version            output version information and exit\n"
-               "  -b, --backup             create backup files (default)\n"
-               "  -B, --no-backup          don't create backup files\n"
-               "  -e, --exit-ask           make save-and-close ask always for confirmation\n"
-               "  -H, --smart-home         go home, then go to first non-blank character\n"
-               "  -i, --ignore-case        make search case insensitive by default\n"
-               "  -k, --keep-lines=<n>     number of lines to keep for PgUp/PgDn\n"
-               "  -m, --max-windows=<n>    max number of windows to show at once\n"
-               "  -n, --indent-step=<n>    number of spaces to add when indenting a block\n"
-//               "  -p, --beep               beep on errors and when cursor goes past extremes\n"
-               "  -s, --search-wrap        make search wrap at end of file\n"
-               "  -u, --auto-unmark        turn off highlighting after any block operation\n"
-               "  -x, --rectangle          rectangular block mode\n"
-               "\nFile options:\n"
-               "  -a, --auto-indent        enable auto indent\n"
-               "  -l, --lmargin=<col>      set left margin\n"
-               "  -r, --rmargin=<col>      set right margin\n"
-               "  -o, --read-only          make buffer read-only\n"
-               "  -O, --overwrite          disable insert mode\n"
-               "  -w, --word-wrap          enable word wrapping\n"
-               "  -W, --no-word-wrap       disable word wrapping\n"
-               "  -2, --two-spaces         make reformat put two spaces after period\n"
-               "\nExit status: 0 for a normal exit, 1 for environmental problems\n"
-               "(invalid command-line options, I/O errors, etc), 2 to indicate an invalid\n"
-               "or unreadable input file, 3 for an internal consistency error (e.g., bug)\n"
-               "which caused moe to panic.\n"
-               "\nReport bugs to bug-moe@gnu.org\n"
-               "Moe home page: http://www.gnu.org/software/moe/moe.html\n"
-               "General help using GNU software: http://www.gnu.org/gethelp\n" );
+  std::printf(
+    "GNU moe is a console text editor for ISO-8859 and ASCII character encodings.\n"
+    "It has a modeless, user-friendly interface, online help, multiple windows,\n"
+    "global search/replace (on all buffers at once), block operations, automatic\n"
+    "indentation, word wrapping, file name completion, directory browser,\n"
+    "duplicate removal from prompt histories, delimiter matching, text conversion\n"
+    "from/to UTF-8, romanization, etc. The file size, line length, number of\n"
+    "buffers, and undo/redo capability are only limited by the amount of memory\n"
+    "available and the size of the address space of your machine.\n"
+    "\nMoe respects your work. By default it won't automatically add, change, or\n"
+    "remove a single byte in your files. Moe is a WYTIWYG (what you type is what\n"
+    "you get) editor.\n"
+    "\nMoe can easily edit thousands of files at the same time.\n"
+    "\nUsage: %s [global_options] [[+[line[,col]]] file_name [file_options]]...\n",
+    invocation_name );
+  std::fputs(
+    "\n'global_options' are a mix of editor options and default 'file_options'.\n"
+    "Most long option names have a negative form: '--backup' '--no-backup'.\n"
+    "\nEach file name may be preceded by '+[line[,column]]' to start the cursor\n"
+    "at the line, and optionally column, specified. 'line' and 'column' are\n"
+    "applied to subsequent files until new values are specified for them.\n"
+    "A '+' alone puts the cursor at end-of-buffer.\n"
+    "\nA hyphen '-' used as a file name argument means standard input.\n"
+    "Directories are recursively descended into.\n"
+    "\nEditor options:\n"
+    "  -h, --help               display this help and exit\n"
+    "  -V, --version            output version information and exit\n"
+    "  -b, --backup             create backup files (default)\n"
+    "  -B, --no-backup          don't create backup files\n"
+    "  -e, --exit-ask           make save-and-close ask always for confirmation\n"
+    "  -H, --smart-home         go home, then go to first non-blank character\n"
+    "  -i, --ignore-case        make search case insensitive by default\n"
+    "  -k, --keep-lines=<n>     number of lines to keep for PgUp/PgDn\n"
+    "  -m, --max-windows=<n>    max number of windows to show at once\n"
+    "  -n, --indent-step=<n>    number of spaces to add when indenting a block\n"
+    "  -p, --preserve-links     preserve (don't break) links when saving files\n"
+    "  -s, --search-wrap        make search wrap at end of file\n"
+    "  -u, --auto-unmark        turn off highlighting after any block operation\n"
+    "  -x, --rectangle          rectangular block mode\n"
+    "\nFile options:\n"
+    "  -a, --auto-indent        enable auto indent\n"
+    "  -l, --lmargin=<col>      set left margin\n"
+    "  -r, --rmargin=<col>      set right margin\n"
+    "  -o, --read-only          make buffer read-only\n"
+    "  -O, --overwrite          disable insert mode\n"
+    "  -w, --word-wrap          enable word wrapping\n"
+    "  -W, --no-word-wrap       disable word wrapping\n"
+    "  -2, --two-spaces         make reformat put two spaces after period\n"
+    "\n*Exit status*\n"
+    "0 for a normal exit, 1 for environmental problems (invalid command-line\n"
+    "options, I/O errors, etc), 2 to indicate an invalid or unreadable input\n"
+    "file, 3 for an internal consistency error (e.g., bug) which caused moe to\n"
+    "panic.\n"
+    "\nReport bugs to bug-moe@gnu.org\n"
+    "Moe home page: http://www.gnu.org/software/moe/moe.html\n"
+    "General help using GNU software: http://www.gnu.org/gethelp\n", stdout );
   }
 
 
@@ -109,9 +112,9 @@ void show_version()
   {
   std::printf( "GNU %s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
-               "This is free software: you are free to change and redistribute it.\n"
-               "There is NO WARRANTY, to the extent permitted by law.\n" );
+  std::fputs( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
+              "This is free software: you are free to change and redistribute it.\n"
+              "There is NO WARRANTY, to the extent permitted by law.\n", stdout );
   }
 
 
@@ -138,6 +141,7 @@ void restore_signals()
   set_signal( SIGILL, SIG_DFL );
   set_signal( SIGINT, SIG_DFL );
   set_signal( SIGPIPE, SIG_DFL );
+  set_signal( SIGQUIT, SIG_DFL );
   set_signal( SIGSEGV, SIG_DFL );
   set_signal( SIGTERM, SIG_DFL );
   set_signal( SIGUSR1, SIG_DFL );
@@ -155,6 +159,7 @@ void set_signals()
   set_signal( SIGILL, termination_handler );
   set_signal( SIGINT, SIG_IGN );
   set_signal( SIGPIPE, SIG_IGN );
+  set_signal( SIGQUIT, SIG_IGN );
   set_signal( SIGSEGV, termination_handler );
   set_signal( SIGTERM, termination_handler );
   set_signal( SIGUSR1, SIG_IGN );
@@ -181,12 +186,12 @@ void maybe_process_config_file()
   if( name.size() )
     {
     name += '/'; name += config_file_name;
-    const int retval = RC::process_rcfile( name );
+    const int retval = RC::process_rcfile( name.c_str() );
     if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return;
     RC::reset();
     }
   name = SYSCONFDIR; name += '/'; name += config_file_name;
-  const int retval = RC::process_rcfile( name );
+  const int retval = RC::process_rcfile( name.c_str() );
   if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return;
   RC::reset();
   std::fprintf( stderr, "Couldn't open '%s'\n", name.c_str() );
