@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,8 @@ bool file_menu( const std::string & prompt, std::string & name );
 int get_filename( const std::string & prompt, std::string & name,
                   const int abort_key = 3 );
 
-bool is_regular( const std::string & name, bool * can_readp = 0 );
+bool is_regular( const std::string & name, bool * can_statp = 0,
+                 bool * can_readp = 0 );
 
 bool make_dirs( const std::string & name );
 
