@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -339,8 +339,10 @@ int Menu::get_filename( const std::string & prompt, std::string & name,
   long size = Screen::get_string( prompt, file_history, abort_key, true );
   if( size > 0 )
     {
-    if( size > 65536 || !is_regular( file_history.back() ) )
-      { size = -1; Screen::show_message( "Not a regular file" ); }
+    bool can_stat = false;
+    if( size > 65536 || !is_regular( file_history.back(), &can_stat ) )
+      { size = -1; Screen::show_message(
+                   can_stat ? "Not a regular file" : "Can't access file" ); }
     else name = file_history.back();
     }
   if( size <= 0 ) name.clear();
@@ -348,17 +350,17 @@ int Menu::get_filename( const std::string & prompt, std::string & name,
   }
 
 
-bool Menu::is_regular( const std::string & name, bool * can_readp )
+bool Menu::is_regular( const std::string & name, bool * can_statp,
+                       bool * can_readp )
   {
   if( name.size() && name.end()[-1] != '/' )
     {
     struct stat st;
     const int i = stat( name.c_str(), &st );
     const mode_t mode = st.st_mode;
-    const bool can_read = ( i == 0 &&
-                            ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
-                              S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
-    if( can_readp ) *can_readp = can_read;
+    if( can_statp ) *can_statp = i == 0;
+    if( can_readp ) *can_readp = i == 0 && ( S_ISBLK( mode ) ||
+                    S_ISCHR( mode ) || S_ISFIFO( mode ) || S_ISSOCK( mode ) );
     if( ( i == 0 && S_ISREG( mode ) ) ||
         ( i == -1 && errno == ENOENT ) ) return true;
     }
