@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +15,9 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <cstdio>
 #include <cctype>
+#include <cstdio>
+#include <cstdlib>
 #include <string>
 #include <vector>
 
@@ -86,11 +87,7 @@ bool ignore_case_change( Buffer & )
 
 
 const char * indent_step_status( const Buffer & )
-  {
-  static char buf[12];		// 12 keeps g++ quiet
-  snprintf( buf, sizeof buf, "%d", RC::editor_options().indent_step() );
-  return buf;
-  }
+  { return format_num3( RC::editor_options().indent_step() ); }
 
 bool indent_step_change( Buffer & )
   {
@@ -109,9 +106,8 @@ bool indent_step_change( Buffer & )
 
 const char * keep_lines_status( const Buffer & )
   {
-  static char buf[12];		// 12 keeps g++ quiet
-  snprintf( buf, sizeof buf, "%d", RC::editor_options().keep_lines() );
-  return buf;
+  const int i = RC::editor_options().keep_lines();
+  return format_num3( std::abs( i ), i < 0 );
   }
 
 bool keep_lines_change( Buffer & )
@@ -130,11 +126,7 @@ bool keep_lines_change( Buffer & )
 
 
 const char * lmargin_status( const Buffer & buffer )
-  {
-  static char buf[12];		// 12 keeps g++ quiet
-  snprintf( buf, sizeof buf, "%d", buffer.options.lmargin() + 1 );
-  return buf;
-  }
+  { return format_num3( buffer.options.lmargin() + 1 ); }
 
 bool lmargin_change( Buffer & buffer )
   {
@@ -152,11 +144,7 @@ bool lmargin_change( Buffer & buffer )
 
 
 const char * max_windows_status( const Buffer & )
-  {
-  static char buf[12];		// 12 keeps g++ quiet
-  snprintf( buf, sizeof buf, "%d", RC::editor_options().max_windows() );
-  return buf;
-  }
+  { return format_num3( RC::editor_options().max_windows() ); }
 
 bool max_windows_change( Buffer & )
   {
@@ -173,6 +161,14 @@ bool max_windows_change( Buffer & )
   }
 
 
+const char * preserve_links_status( const Buffer & )
+  { return RC::editor_options().preserve_links ? "ON" : "OFF"; }
+
+bool preserve_links_change( Buffer & )
+  { RC::editor_options().preserve_links = !RC::editor_options().preserve_links;
+    return true; }
+
+
 const char * read_only_status( const Buffer & buffer )
   { return buffer.options.read_only ? "ON" : "OFF"; }
 
@@ -181,11 +177,7 @@ bool read_only_change( Buffer & buffer )
 
 
 const char * rmargin_status( const Buffer & buffer )
-  {
-  static char buf[12];		// 12 keeps g++ quiet
-  snprintf( buf, sizeof buf, "%d", buffer.options.rmargin() + 1 );
-  return buf;
-  }
+  { return format_num3( buffer.options.rmargin() + 1 ); }
 
 bool rmargin_change( Buffer & buffer )
   {
@@ -262,6 +254,7 @@ Entry table[] = {
   { 'M', "Max windows to show ", &max_windows_status, &max_windows_change },
   { 'N', "Indent step ", &indent_step_status, &indent_step_change },
   { 'O', "Read only ", &read_only_status, &read_only_change },
+  { 'P', "Preserve links ", &preserve_links_status, &preserve_links_change },
 //  { 'P', "Beeps ", &beep_status, &beep_change },
   { 'R', "Right margin ", &rmargin_status, &rmargin_change },
   { 'S', "Search wrap ", &search_wrap_status, &search_wrap_change },
@@ -378,7 +371,7 @@ void Menu::options_menu( Buffer & buffer, const int abort_key )
     if( entry.change && entry.change( buffer ) )
       {
       const char * const new_st = entry.status ? entry.status( buffer ) : 0;
-      if( new_st ) msg += new_st;	// old_st,new_st point to the same buf
+      if( new_st ) msg += new_st;
       if( msg.size() )
         { Screen::show_status_lines( &buffer ); Screen::show_message( msg ); }
       }
