@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,8 +44,8 @@
 #endif
 
 // order of application of file options by place of specification:
-// 1 options specified in moe.conf file as 'default file options'.
-// 2 options specified in moe.conf file as 'file name dependent options'.
+// 1 options specified in file moe.conf as 'default file options'.
+// 2 options specified in file moe.conf as 'file name dependent options'.
 // 3 options specified in command line as 'default file options'.
 // 4 options specified in command line as 'file options'.
 // (4 is not applied to files opened during the editing sesion).
@@ -92,51 +92,54 @@ enum Optcode
   {
   auto_unmark = 'u', backup = 'b', no_backup = 'B', /*beep = 'p',*/
   exit_ask = 'e', ignore_case = 'i', indent_step = 'n', keep_lines = 'k',
-  max_windows = 'm', rectangle_mode = 'x', search_wrap = 's', smart_home = 'H',
-  auto_indent = 'a', lmargin = 'l', rmargin = 'r', overwrite = 'O',
-  read_only = 'o', two_spaces = '2', word_wrap = 'w', no_word_wrap = 'W',
+  max_windows = 'm', preserve_links = 'p', rectangle_mode = 'x',
+  search_wrap = 's', smart_home = 'H', auto_indent = 'a', lmargin = 'l',
+  rmargin = 'r', overwrite = 'O', read_only = 'o', two_spaces = '2',
+  word_wrap = 'w', no_word_wrap = 'W',
   no_auto_unmark = 256, /*no_beep,*/ no_exit_ask, no_ignore_case,
-  no_rectangle_mode, no_search_wrap, no_smart_home,
+  no_preserve_links, no_rectangle_mode, no_search_wrap, no_smart_home,
   no_auto_indent, no_overwrite, no_read_only, no_two_spaces
   };
 
 const Arg_parser::Option options[] =
   {
-  { 'h',               "help",           Arg_parser::no  },
-  { 'V',               "version",        Arg_parser::no  },
-  { auto_unmark,       "auto-unmark",    Arg_parser::no  },
-  { no_auto_unmark,    "no-auto-unmark", Arg_parser::no  },
-  { backup,            "backup",         Arg_parser::no  },
-  { no_backup,         "no-backup",      Arg_parser::no  },
-//  { beep,              "beep",           Arg_parser::no  },
-//  { no_beep,           "no-beep",        Arg_parser::no  },
-  { exit_ask,          "exit-ask",       Arg_parser::no  },
-  { no_exit_ask,       "no-exit-ask",    Arg_parser::no  },
-  { ignore_case,       "ignore-case",    Arg_parser::no  },
-  { no_ignore_case,    "no-ignore-case", Arg_parser::no  },
-  { indent_step,       "indent-step",    Arg_parser::yes },
-  { keep_lines,        "keep-lines",     Arg_parser::yes },
-  { max_windows,       "max-windows",    Arg_parser::yes },
-  { rectangle_mode,    "rectangle",      Arg_parser::no  },
-  { no_rectangle_mode, "no-rectangle",   Arg_parser::no  },
-  { search_wrap,       "search-wrap",    Arg_parser::no  },
-  { no_search_wrap,    "no-search-wrap", Arg_parser::no  },
-  { smart_home,        "smart-home",     Arg_parser::no  },
-  { no_smart_home,     "no-smart-home",  Arg_parser::no  },
-  { auto_indent,       "auto-indent",    Arg_parser::no  },
-  { no_auto_indent,    "no-auto-indent", Arg_parser::no  },
-  { lmargin,           "lmargin",        Arg_parser::yes },
-  { rmargin,           "rmargin",        Arg_parser::yes },
-  { overwrite,         "overwrite",      Arg_parser::no  },
-  { no_overwrite,      "no-overwrite",   Arg_parser::no  },
-  { read_only,         "read-only",      Arg_parser::no  },
-  { no_read_only,      "no-read-only",   Arg_parser::no  },
-  { two_spaces,        "two-spaces",     Arg_parser::no  },
-  { no_two_spaces,     "no-two-spaces",  Arg_parser::no  },
-  { word_wrap,         "word-wrap",      Arg_parser::no  },
-  { no_word_wrap,      "no-word-wrap",   Arg_parser::no  },
-  { '1',               "orphan",         Arg_parser::no  },	// ignored
-  { 0,                 0,                Arg_parser::no  } };
+  { 'h',               "help",              Arg_parser::no  },
+  { 'V',               "version",           Arg_parser::no  },
+  { auto_unmark,       "auto-unmark",       Arg_parser::no  },
+  { no_auto_unmark,    "no-auto-unmark",    Arg_parser::no  },
+  { backup,            "backup",            Arg_parser::no  },
+  { no_backup,         "no-backup",         Arg_parser::no  },
+//  { beep,              "beep",              Arg_parser::no  },
+//  { no_beep,           "no-beep",           Arg_parser::no  },
+  { exit_ask,          "exit-ask",          Arg_parser::no  },
+  { no_exit_ask,       "no-exit-ask",       Arg_parser::no  },
+  { ignore_case,       "ignore-case",       Arg_parser::no  },
+  { no_ignore_case,    "no-ignore-case",    Arg_parser::no  },
+  { indent_step,       "indent-step",       Arg_parser::yes },
+  { keep_lines,        "keep-lines",        Arg_parser::yes },
+  { max_windows,       "max-windows",       Arg_parser::yes },
+  { preserve_links,    "preserve-links",    Arg_parser::no  },
+  { no_preserve_links, "no-preserve-links", Arg_parser::no  },
+  { rectangle_mode,    "rectangle",         Arg_parser::no  },
+  { no_rectangle_mode, "no-rectangle",      Arg_parser::no  },
+  { search_wrap,       "search-wrap",       Arg_parser::no  },
+  { no_search_wrap,    "no-search-wrap",    Arg_parser::no  },
+  { smart_home,        "smart-home",        Arg_parser::no  },
+  { no_smart_home,     "no-smart-home",     Arg_parser::no  },
+  { auto_indent,       "auto-indent",       Arg_parser::no  },
+  { no_auto_indent,    "no-auto-indent",    Arg_parser::no  },
+  { lmargin,           "lmargin",           Arg_parser::yes },
+  { rmargin,           "rmargin",           Arg_parser::yes },
+  { overwrite,         "overwrite",         Arg_parser::no  },
+  { no_overwrite,      "no-overwrite",      Arg_parser::no  },
+  { read_only,         "read-only",         Arg_parser::no  },
+  { no_read_only,      "no-read-only",      Arg_parser::no  },
+  { two_spaces,        "two-spaces",        Arg_parser::no  },
+  { no_two_spaces,     "no-two-spaces",     Arg_parser::no  },
+  { word_wrap,         "word-wrap",         Arg_parser::no  },
+  { no_word_wrap,      "no-word-wrap",      Arg_parser::no  },
+  { '1',               "orphan",            Arg_parser::no  },	// ignored
+  { 0,                 0,                   Arg_parser::no  } };
 
 
 bool is_editor_option( const int code )
@@ -196,6 +199,8 @@ bool set_option_from_code( const Arg_parser & parser, const int argind,
       if( !parse_int( arg, n ) ) errcode = 1;
       else if( !editor_options_.set_max_windows( n ) ) errcode = 2;
       break;
+    case preserve_links:    editor_options_.preserve_links = true; break;
+    case no_preserve_links: editor_options_.preserve_links = false; break;
     case rectangle_mode:    editor_options_.rectangle_mode = true; break;
     case no_rectangle_mode: editor_options_.rectangle_mode = false; break;
     case search_wrap:       editor_options_.search_wrap = true; break;
@@ -241,7 +246,7 @@ bool set_option_from_code( const Arg_parser & parser, const int argind,
 int parse_line_col( const char * const ptr, long & line, long & col )
   {
   if( ptr[0] == '+' && ptr[1] == 0 ) { line = col = LONG_MAX - 1; return 1; }
-  int c = parse_long( ptr, line );
+  int c = parse_long( ptr, line, false );
   if( c <= 0 || --line < 0 ) return 0;
   if( ptr[c] )
     {
@@ -350,7 +355,7 @@ bool add_handles_recursively( std::string name, const long line, const long col,
     DIR * const dirp = opendir( dirname.c_str() );
     if( !dirp ) { read_error_found = true; continue; }
     std::vector< std::string > name_vector;
-    const bool isdot = ( dirname == "." );
+    const bool isdot = dirname == ".";
     while( true )
       {
       const struct dirent * const entryp = readdir( dirp );
@@ -404,6 +409,15 @@ bool add_handles_recursively( std::string name, const long line, const long col,
   return true;
   }
 
+
+int chvalue( const unsigned char ch )
+  {
+  if( ch >= '0' && ch <= '9' ) return ch - '0';
+  if( ch >= 'A' && ch <= 'Z' ) return ch - 'A' + 10;
+  if( ch >= 'a' && ch <= 'z' ) return ch - 'a' + 10;
+  return 255;
+  }
+
 } // end namespace RC
 
 
@@ -437,15 +451,29 @@ int RC::parse_int( const char * const ptr, int & result )
   }
 
 // Return the number of bytes read, or 0 if error.
-int RC::parse_long( const char * const ptr, long & result )
+int RC::parse_long( const char * const ptr, long & result, const bool last )
   {
-  if( !ptr || !ptr[0] ) return 0;
-  char * tail;
-  errno = 0;
-  const long tmp = std::strtol( ptr, &tail, 0 );
-  if( errno || tail == ptr || tmp < -LONG_MAX ) return 0;
-  result = tmp;
-  const int c = tail - ptr;
+  int c = 0, base;
+  while( ISO_8859::isspace( ptr[c] ) ) ++c;
+  const bool minus = ptr[c] == '-';
+  if( minus || ptr[c] == '+' ) ++c;
+  if( !std::isdigit( ptr[c] ) ) return 0;
+
+  if( ptr[c] != '0' ) base = 10;			// decimal
+  else if( ptr[c+1] == 'x' || ptr[c+1] == 'X' ) { base = 16; c += 2; }	// hex
+  else base = 8;					// octal or 0
+  const int dpg = ( base != 16 ) ? 3 : 2;	// min digits per group
+  int dig = dpg - 1;	// digits in current group, first may have 1 digit
+  result = 0;
+  for( ; ptr[c]; ++c )
+    {
+    if( ptr[c] == '_' ) { if( dig < dpg ) break; else { dig = 0; continue; } }
+    const int val = chvalue( ptr[c] ); if( val >= base ) break; else ++dig;
+    if( ( LONG_MAX - val ) / base < result ) return 0;
+    result = result * base + val;
+    }
+  if( ( last && ptr[c] != 0 ) || dig < dpg ) return 0;
+  if( minus ) result = -result;
   return c;
   }
 
@@ -471,12 +499,12 @@ int RC::parse_relative_long( const char * const ptr, long & result )
 
 
 // Return 0 if success, 1 if file not found, 2 if syntax error.
-int RC::process_rcfile( const std::string & name )
+int RC::process_rcfile( const char * const name )
   {
-  FILE * const f = std::fopen( name.c_str(), "r" );
+  FILE * const f = std::fopen( name, "r" );
   if( !f ) return 1;
 
-  std::fprintf( stderr, "Processing '%s'... ", name.c_str() );
+  std::fprintf( stderr, "Processing '%s'... ", name );
   std::fflush( stderr );
 
   const char delimiters[] = " \t\r\n";
@@ -484,7 +512,7 @@ int RC::process_rcfile( const std::string & name )
   int regex_status = 0;		// 1 = got regex name, 2 got regex option
   enum { bufsize = 1024 };
   char buf[bufsize];
-  for( int line = 1; ; ++line )
+  for( unsigned line = 1; ; ++line )
     {
     if( !std::fgets( buf, bufsize, f ) ) break;
     int len = 0;
@@ -497,8 +525,7 @@ int RC::process_rcfile( const std::string & name )
         if( ch == EOF || ch == '\n' ) break;
         }
       if( !retval ) std::fputc( '\n', stderr );
-      std::fprintf( stderr, "%s %d: error: Line too long\n",
-                    name.c_str(), line );
+      std::fprintf( stderr, "%s %u: error: Line too long\n", name, line );
       retval = 2;
       }
     if( len == 0 || ISO_8859::isspace( buf[0] ) ) continue;
@@ -515,8 +542,7 @@ int RC::process_rcfile( const std::string & name )
       const char * const opt = std::strtok( buf, delimiters );
       const char * const arg = std::strtok( 0, delimiters );
       char msg[80];
-      snprintf( msg, sizeof msg, "%s%s %d: ",
-                retval ? "" : "\n", name.c_str(), line );
+      snprintf( msg, sizeof msg, "%s%s %u: ", retval ? "" : "\n", name, line );
       const Arg_parser parser( opt, arg, options );
       if( parser.error().size() || !parser.arguments() || !parser.code( 0 ) )
         { std::fprintf( stderr, "%s%s\n", msg, parser.error().c_str() );
@@ -540,9 +566,7 @@ int RC::process_rcfile( const std::string & name )
 // Return 0 if success, 1 if invalid option, 2 if a file can't be read.
 int RC::process_options()
   {
-  std::fputs( "Processing options... ", stderr );
-  std::fflush( stderr );
-
+  std::fputs( "Processing options... ", stderr ); std::fflush( stderr );
   int argind = 0;
   for( ; argind < parserp->arguments(); ++argind )
     {						// process "global" options
@@ -551,6 +575,7 @@ int RC::process_options()
                                default_file_options, "\n", true ) ) return 1;
     }
 
+  std::fputs( "done\nLoading files... ", stderr ); std::fflush( stderr );
   long line = 0, col = 0;
   bool first_post = true;
   bool read_error_found = false;
@@ -583,12 +608,12 @@ int RC::process_options()
       if( !set_option_from_code( *parserp, argind, file_options,
                                  first_post ? "\n" : "" ) ) return 1;
       }
-    bool can_read = false;
-    const bool is_stdin = ( name == "-" );
+    bool can_stat = false, can_read = false;
+    const bool is_stdin = name == "-";
     if( is_stdin && ( stdin_used || isatty( fileno( stdin ) ) ) )
       ;		// ignore stdin if is repeated or is a terminal
     else if( is_stdin || name.empty() ||
-             Menu::is_regular( name, &can_read ) || can_read )
+             Menu::is_regular( name, &can_stat, &can_read ) || can_read )
       {
       try {
         const int i = Bufhandle_vector::find_or_add_handle( name, -1, line,
@@ -606,8 +631,9 @@ int RC::process_options()
                                        first_post, read_error_found ) )
       {
       if( first_post ) { first_post = false; std::fputc( '\n', stderr ); }
-      std::fprintf( stderr, "warning: '%s' is not a supported file type\n", name.c_str() );
-      unsupported_found = true;
+      std::fprintf( stderr, "warning: '%s': %s\n", name.c_str(),
+                    can_stat ? "Unsupported file type" : "Can't access file" );
+      if( can_stat ) unsupported_found = true; else read_error_found = true;
       }
     if( is_stdin ) stdin_used = true;
     }
@@ -636,7 +662,7 @@ int RC::process_options()
 
 int RC::read_options( const int argc, const char * const argv[] )
   {
-  parserp = new Arg_parser( argc, argv, options, true );
+  parserp = new Arg_parser( argc, argv, options, Arg_parser::in_order );
   if( parserp->error().size() )				// bad option
     { show_error( parserp->error().c_str(), 0, true ); return 1; }
   for( int i = 0; i < parserp->arguments(); ++i )
