@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,13 +23,13 @@ class Options
 public:
   int show_code_lpos;			// 1 show code, 2 show lpos
   bool auto_unmark, backup, /*beep,*/ exit_ask, ignore_case;
-  bool rectangle_mode, search_wrap, smart_home;
+  bool preserve_links, rectangle_mode, search_wrap, smart_home;
 
   Options()
     : indent_step_( 2 ), keep_lines_( -1 ), max_windows_( 0 ),
     show_code_lpos( 0 ),
     auto_unmark( false ), backup( true ), /*beep( false ),*/ exit_ask( false ),
-    ignore_case( false ), rectangle_mode( false ),
+    ignore_case( false ), preserve_links( false ), rectangle_mode( false ),
     search_wrap( false ), smart_home( true ) {}
 
   int indent_step() const { return indent_step_; }
@@ -55,11 +55,11 @@ bool issigned( const char * ptr );
 
 // Return the number of bytes read, or 0 if error.
 int parse_int( const char * const ptr, int & result );
-int parse_long( const char * const ptr, long & result );
+int parse_long( const char * const ptr, long & result, const bool last = true );
 int parse_relative_long( const char * const ptr, long & result );
 
 // Return 0 if success, 1 if file not found, 2 if syntax error.
-int process_rcfile( const std::string & name );
+int process_rcfile( const char * const name );
 
 // Return 0 if success, 1 if invalid option, 2 if a file can't be read.
 int process_options();
@@ -68,9 +68,12 @@ int read_options( const int argc, const char * const argv[] );
 
 void reset();
 
-// Defined in main.cc
+// defined in main.cc
 const std::string & home_directory();
 void show_error( const char * const msg, const int errcode = 0,
                  const bool help = false );
 
 } // end namespace RC
+
+// defined in window.cc
+const char * format_num3( unsigned long long num, const bool negative = false );
