@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -213,8 +213,9 @@ int match_regex( const Basic_buffer & buffer, Point & p,
     ch1 = regex[i];
     switch( ch1 )	// ch1 is first (maybe only) of a escape secuence
       {
-      case '^': if( p != buffer.bol( p ) || p == buffer.eof() ) retval = 1; break;
-      case '$': if( p != buffer.eol( p ) ) retval = 1; break;
+      case '^': if( p != buffer.bol( p ) ) retval = 1; break;
+      case '$': if( p != buffer.eol( p ) ||
+                    ( p == buffer.eof() && p.col == 0 ) ) {retval = 1;} break;
       case '<': if( !buffer.pisbow( p ) ) retval = 1; break;
       case '>': if( !buffer.piseow( p ) ) retval = 1; break;
       case '?': {
@@ -359,7 +360,7 @@ int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
   if( bs > 0 )
     {
     const std::string prefix( regex, 0, bs );	// ordinary characters
-    const bool fixed_string = ( prefix.size() == regex.size() );
+    const bool fixed_string = prefix.size() == regex.size();
     if( backward ) while( true )
       {
       if( !buffer.rfind_text( p1, prefix, icase ) ) return 1;
@@ -468,11 +469,10 @@ bool Regex::match_filename( const std::string & regex, const std::string & name,
     switch( ch1 )
       {
       case '?': ++i2; break;
-      case '[': {
-                Set set( regex, i1 );		// set of characters to match
-                if( !set.includes( name[i2++] ) ) return false;
-                i1 += set.parsed_len() - 1;
-                } break;
+      case '[': { Set set( regex, i1 );		// set of characters to match
+                  if( set.parsed_len() > 0 )	// valid set
+                    { if( !set.includes( name[i2++] ) ) return false;
+                      i1 += set.parsed_len() - 1; break; } }	// fall through
       default: if( ch1 != (unsigned char)name[i2++] ) return false;
       }
     }
