@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -94,7 +94,7 @@ void out_raw_char( unsigned char ch, chtype atr )
   {
   if( ch < 32 ) { ch += '@'; atr |= A_UNDERLINE; }
   else if( ch >= 0x7F && ch < 0xA0 )
-    { ch = ( ch + '@' - 0x80 ); atr |= ( A_REVERSE | A_UNDERLINE ); }
+    { ch = ch + '@' - 0x80; atr |= A_REVERSE | A_UNDERLINE; }
   waddch( stdscr, ch | atr );
   }
 
@@ -360,16 +360,19 @@ const char * user_data( const std::string & s, const int base )
   if( s[last] == '+' ) { --last; big_endian = true; }
 
   if( base == 16 )
-    for( int i = last; i >= 2; i -= 2 )
+    for( int i = last; i >= 2; )
       {
-      const int lsd = ISO_8859::xvalue( s[i] );
-      const int msd = ( i > 2 ) ? ISO_8859::xvalue( s[i-1] ) : 0;
+      if( i > 2 && i < last && s[i] == '_' ) --i;
+      const int lsd = ISO_8859::xvalue( s[i--] );
+      if( i > 2 && s[i] == '_' ) --i;
+      const int msd = ( i >= 2 ) ? ISO_8859::xvalue( s[i--] ) : 0;
       if( lsd < 0 || msd < 0 ) return "Invalid hexadecimal data";
       buf.push_back( ( msd << 4 ) | lsd );
       }
   else
     for( int i = 0; i <= last; ++i )
       {
+      if( i > 0 && i < last && s[i] == '_' ) continue;
       if( base == 10 && !std::isdigit( (unsigned char)s[i] ) )
         return "Invalid decimal data";
       if( base == 8 &&  !ISO_8859::isodigit( s[i] ) )
@@ -377,7 +380,7 @@ const char * user_data( const std::string & s, const int base )
       int carry = s[i] - '0';
       for( unsigned j = 0; j < buf.size(); ++j )
         {
-        buf[j] = ( buf[j] * base ) + carry;
+        buf[j] = buf[j] * base + carry;
         carry = buf[j] / 256;
         buf[j] %= 256;
         }
@@ -535,10 +538,10 @@ extern "C" void Screen::clock_handler( int sig )
     if( t )
       {
       const int h = t->tm_hour;
-      clock_string_[0] = ( h < 10 ) ? ' ' : ( h / 10 ) + '0';
-      clock_string_[1] = ( h % 10 ) + '0';
-      clock_string_[3] = ( t->tm_min / 10 ) + '0';
-      clock_string_[4] = ( t->tm_min % 10 ) + '0';
+      clock_string_[0] = ( h < 10 ) ? ' ' : h / 10 + '0';
+      clock_string_[1] = h % 10 + '0';
+      clock_string_[3] = t->tm_min / 10 + '0';
+      clock_string_[4] = t->tm_min % 10 + '0';
       }
     }
   if( sig && time_change )
