@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,43 @@
 #include "window.h"
 
 
+namespace {
+
+// len of num printed with underscores: [-]1_234_567
+int num_len( const long num )
+  {
+  int len = 0;
+  for( long i = num; i != 0; i /= 10 ) ++len;
+  if( len >= 5 ) len += ( len - 1 ) / 3;
+  if( len == 0 || num < 0 ) ++len;
+  return len;
+  }
+
+} // end namespace
+
+
+// separate numbers of 5 or more digits in groups of 3 digits using '_'
+const char * format_num3( unsigned long long num, const bool negative )
+  {
+  enum { buffers = 8, bufsize = 4 * sizeof num };
+  static char buffer[buffers][bufsize];	// circle of buffers for printf
+  static int current = 0;
+
+  char * const buf = buffer[current++]; current %= buffers;
+  char * p = buf + bufsize - 1;		// fill the buffer backwards
+  *p = 0;				// terminator
+  const bool split = num >= 10000;
+
+  for( int i = 0; ; )
+    {
+    *(--p) = num % 10 + '0'; num /= 10; if( num == 0 ) break;
+    if( split && ++i >= 3 ) { i = 0; *(--p) = '_'; }
+    }
+  if( negative ) *(--p) = '-';
+  return p;
+  }
+
+
 Window::Window( Buffer_handle & bh, const int tl, const int h, const bool center )
   : bhp( &bh ), top_line_( tl ), height_( h )
   {
@@ -73,11 +110,9 @@ void Window::goto_home()
   const Buffer & buffer = bhp->buffer();
   if( bhp->cursor != bhp->pcursor || p != buffer.bol( p ) )
     p = buffer.bol( p );
-  else
-    {
-    if( !RC::editor_options().smart_home || p == buffer.bot( p ) ) return;
+  else if( RC::editor_options().smart_home && p != buffer.bot( p ) )
     p = buffer.bot( p );
-    }
+  else return;
   update_points( p );
   }
 
@@ -200,7 +235,7 @@ const char * Window::goto_mark( const int i )
       {
       int bufsize = 0;
       char * const buf = Screen::msg_line_buf( &bufsize );
-      snprintf( buf, bufsize, "Mark %d not set", i );
+      snprintf( buf, bufsize, "Mark %u not set", i );
       return buf;
       }
     }
@@ -329,7 +364,7 @@ const char * Window::extend_marks()
     {
     int bufsize = 0;
     char * const buf = Screen::msg_line_buf( &bufsize );
-    snprintf( buf, bufsize, "%d new mark(s) set", c );
+    snprintf( buf, bufsize, "%u new mark(s) set", c );
     return buf;
     }
   return "No new marks set";
@@ -343,7 +378,7 @@ const char * Window::set_mark( const int i )
     bhp->buffer().mark[i] = bhp->pointer;
     int bufsize = 0;
     char * const buf = Screen::msg_line_buf( &bufsize );
-    snprintf( buf, bufsize, "Mark %d set", i );
+    snprintf( buf, bufsize, "Mark %u set", i );
     return buf;
     }
   return 0;
@@ -368,20 +403,21 @@ const char * Window::show_character_info() const
   const int ch = buffer[p];
 
   if( ch < 0 && p != buffer.eof() )
-    snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %ld  Pos %ld",
-              p.line + 1, p.col + 1 );
+    snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %s  Pos %s",
+              format_num3( p.line + 1 ), format_num3( p.col + 1 ) );
   else
     {
     int percent = 0;
     long offset = buffer.character_offset( p, percent );
-    const int len = snprintf( buf, bufsize, "Line %ld  Pos %ld  Offset %ld(0x%lX)  %d%%  ",
-                              p.line + 1, p.col + 1, offset, offset, percent );
+    const int len = snprintf( buf, bufsize, "Line %s  Pos %s  Offset %s(0x%s)  %u%%  ",
+                      format_num3( p.line + 1 ), format_num3( p.col + 1 ),
+                      format_num3( offset ), format_hex4( offset ), percent );
     if( len > 0 && len < bufsize )
       {
       if( ch >= 0 )
-        snprintf( buf + len, bufsize - len, "Code %d(0x%02X/0%o)", ch, ch ,ch );
+        snprintf( buf + len, bufsize - len, "%u(0x%02X/0%o)", ch, ch ,ch );
       else
-        snprintf( buf + len, bufsize - len, "(End Of File)" );
+        snprintf( buf + len, bufsize - len, "End Of File" );
       }
     }
   return buf;
@@ -413,16 +449,16 @@ const char * Window::show_multichar_value( const int size, const bool big_endian
     else
       for( int i = size - 1; i >= 0; --i )
         { value <<= 8; value += (unsigned char)text[i]; }
-    snprintf( buf, bufsize, "Off %ld(0x%lX)  %s_en_%d %llu(0x%0*llX)",
-              offset, offset, big_endian ? "Big" : "Lit",
-              size, value, 2 * size, value );
+    snprintf( buf, bufsize, "Off %s(0x%s)  %s(0x%s) %cE%u",
+            format_num3( offset ), format_hex4( offset ), format_num3( value ),
+            format_hex4( value ), big_endian ? 'B' : 'L', size );
     }
   else if( sz > 0 )
-    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (Only %d Characters Left)",
-              offset, offset, percent, sz );
+    snprintf( buf, bufsize, "Offset %s(0x%s)  %u%%  (Only %u Characters Left)",
+              format_num3( offset ), format_hex4( offset ), percent, sz );
   else
-    snprintf( buf, bufsize, "Offset %ld(0x%lX)  %d%%  (End Of File)",
-              offset, offset, percent );
+    snprintf( buf, bufsize, "Offset %s(0x%s)  %u%%  (End Of File)",
+              format_num3( offset ), format_hex4( offset ), percent );
   return buf;
   }
 
@@ -450,17 +486,17 @@ const char * Window::show_utf8_code()
   if( code >= 0 )
     {
     if( p != bhp->pointer ) update_points( p );
-    snprintf( buf, bufsize, "UTF-8 Code %d(U%04X)  Length %d",
+    snprintf( buf, bufsize, "UTF-8 Code %u(U%04X)  Length %u",
               code, code, len );
     }
   else if( byte_code >= 0 )
-    snprintf( buf, bufsize, "Invalid UTF-8 Byte %d(0x%02X)",
+    snprintf( buf, bufsize, "Invalid UTF-8 Byte %u(0x%02X)",
               byte_code, byte_code );
   else if( p == buffer.eof() )
     snprintf( buf, bufsize, "(End Of File)" );
   else
-    snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %ld  Pos %ld",
-              p.line + 1, p.col + 1 );
+    snprintf( buf, bufsize, "*** Error: Bad Pointer: Line %s  Pos %s",
+              format_num3( p.line + 1 ), format_num3( p.col + 1 ) );
   return buf;
   }
 
@@ -476,8 +512,8 @@ void Window::show_status_line( const char * const prefix ) const
   char * const buf = Screen::status_line_buf( &bufsize );
   int offset = bufsize - 41;
   const long line = bh.pcursor.line + 1, col = bh.pcursor.col + 1;
-  for( long i = line; i > 9999; i /= 10 ) --offset;
-  for( long i = col; i > 999; i /= 10 ) --offset;
+  if( line > 9999 ) offset -= num_len( line ) - 4;
+  if( col > 999 ) offset -= num_len( col ) - 3;
   const char * name = buffer.nename().c_str();
   const int overlap = buffer.nename().size() + 11 - offset;
   if( overlap > 0 ) name += overlap;
@@ -493,8 +529,8 @@ void Window::show_status_line( const char * const prefix ) const
                             modified ? " (Modified)" : "" );
   if( len > 0 && len < offset ) std::memset( buf + len, ' ', offset - len );
   snprintf( buf + offset, bufsize - offset,
-            "  Line %-4ld Col %-3ld %5s   F1 for help",
-            line, col, Screen::clock_string() );
+            "  Line %-4s Col %-3s %5s   F1 for help",
+            format_num3( line ), format_num3( col ), Screen::clock_string() );
   if( RC::editor_options().show_code_lpos )
     {
     offset = bufsize - 13;
@@ -502,7 +538,7 @@ void Window::show_status_line( const char * const prefix ) const
       {
       const int ch = buffer[bh.pointer];
       if( ch >= 0 )
-        snprintf( buf + offset, bufsize - offset, "%3d(0x%02X)", ch, ch );
+        snprintf( buf + offset, bufsize - offset, "%3u(0x%02X)", ch, ch );
       else
         snprintf( buf + offset, bufsize - offset, "End of file" );
       }
@@ -510,7 +546,7 @@ void Window::show_status_line( const char * const prefix ) const
       {
       const int lpos = ( line <= 1 ) ? 0 : ( line >= buffer.lines() ) ? 100 :
                        ( 100LL * line ) / buffer.lines();
-      snprintf( buf + offset, bufsize - offset, "Lpos %d%%", lpos );
+      snprintf( buf + offset, bufsize - offset, "Lpos %u%%", lpos );
       }
     }
   Screen::out_line( buf, top_line_, false, 'i' );
