@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2025 Antonio Diaz Diaz.
+   Copyright (C) 2005-2026 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -195,13 +195,13 @@ bool insert_new_year( const Buffer & buffer, const Point & p0,
     else outbuf.pputc( pt, ' ' );
     }
 
-  if( !change ) change = ( outbuf.last_line() != ( ph2.line - p0.line ) );
+  if( !change ) change = outbuf.last_line() != ph2.line - p0.line;
   if( !change )
     {
     std::string s1, s2;
     buffer.to_string( p0, ph2, s1 );
     outbuf.to_string( outbuf.bof(), outbuf.eof(), s2 );
-    change = ( s1 != s2 );
+    change = s1 != s2;
     }
   return change;			// new_year inserted or list modified
   }
@@ -270,7 +270,7 @@ int Window_vector::windows() { return data.size(); }
 int Window_vector::load( int new_size )
   {
   const int handles = Bufhandle_vector::handles();
-  const bool toggle = ( new_size < 0 && handles > 1 );
+  const bool toggle = new_size < 0 && handles > 1;
 
   if( !handles ) { curwin_ = 0; data.clear(); return 0; }
   if( new_size == 0 ) new_size = windows();
@@ -508,7 +508,7 @@ const char * Window_vector::add_char( const int key, const bool force )
     }
   int bufsize = 0;
   char * const buf = Screen::msg_line_buf( &bufsize );
-  snprintf( buf, bufsize, "Key code %d(0x%02X) ignored", key, key );
+  snprintf( buf, bufsize, "Key code %u(0x%02X) ignored", key, key );
   return buf;
   }
 
@@ -547,11 +547,17 @@ void Window_vector::copy_block()
 
 void Window_vector::delete_block()
   {
-  Buffer & buffer = curwin().buffer();
+  Window & w = curwin();
+  Buffer & buffer = w.buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
+    return;
+  const Point p = w.buffer_handle().pointer;
+  const long left = w.buffer_handle().top_left.col;
+  const bool move = p > Block::begin() && p.line <= Block::end().line &&
+                    buffer.to_cursor( Block::begin() ).col < left;
 
-  if( !no_block() && !no_block_in_this_file( buffer ) &&
-      !read_only( buffer ) && Block::delete_block() )
-    Screen::repaint( &buffer );
+  if( Block::delete_block() )
+    { if( move ) w.to_left(); Screen::repaint( &buffer ); }
   }
 
 
@@ -635,12 +641,13 @@ void Window_vector::indent_block()
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return;
 
+  const long last_line = Block::last_mline();
   Point p = Block::begin();
   bool first_post = true;
   const bool ow = buffer.options.overwrite;
   buffer.options.overwrite = false;
   buffer.reset_appendable();
-  while( buffer.bol( p ) < Block::end() )
+  while( p.line <= last_line )
     {
     p = buffer.bot( p );
     if( p.col > 0 || p < buffer.eol( p ) )
@@ -664,9 +671,10 @@ const char * Window_vector::unindent_block()
   if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
       read_only( buffer ) ) return 0;
 
+  const long last_line = Block::last_mline();
   Point p = Block::begin();
   bool fail = false;
-  while( buffer.bol( p ) < Block::end() && !fail )
+  while( p.line <= last_line && !fail )
     {
     if( buffer.eol( p ).col > 0 &&
         buffer.to_cursor( buffer.bot( p ) ).col < RC::editor_options().indent_step() )
@@ -680,7 +688,7 @@ const char * Window_vector::unindent_block()
   buffer.options.overwrite = false;
   buffer.reset_appendable();
   p = Block::begin();
-  while( buffer.bol( p ) < Block::end() )
+  while( p.line <= last_line )
     {
     const long target_col = buffer.to_cursor( buffer.bot( p ) ).col -
                             RC::editor_options().indent_step();
@@ -1013,7 +1021,7 @@ const char * Window_vector::remove_utf8_out_of_range()
   Block::bufferp()->to_string( Block::begin(), Block::end(), in );
   long idx;
   const int ret = Encoding::remove_utf8_out_of_range( in, out, &idx );
-  if( ret == 0 ) return "Text in block is already decoded";
+  if( ret == 0 ) return "UTF-8 text in block is already in range";
   if( ret < 0 )
     {
     Block::disable_marking();
@@ -1307,8 +1315,8 @@ const char * Window_vector::search( const int abort_key, const bool again,
     { const int ret = Regex::find( w.buffer(), p1, p2, search_string,
                                    regex_pieces, icase, backward );
     if( ret == 2 ) { msg = "Search error: check syntax"; break; }
-    found = ( ret == 0 ); }
-    empty_replace = ( found && ( p1 == p2 ) );
+    found = ret == 0; }
+    empty_replace = found && p1 == p2;
     if( found && block &&
         ( p1 < Block::begin() || p1 >= Block::end() || p2 > Block::end() ) )
       found = false;
